@@ -19,8 +19,8 @@ from cStringIO import StringIO
 from sphinx.util import format_exception_cut_frames, save_traceback
 from sphinx.util.console import darkred, nocolor
 
-__revision__ = '$Revision: 62710 $'
-__version__ = '0.3'
+__revision__ = '$Revision: 64484 $'
+__version__ = '0.4'
 
 
 def usage(argv, msg=None):
@@ -135,9 +135,12 @@ def main(argv=sys.argv):
             app.builder.build_update()
     except KeyboardInterrupt:
         # catches BaseExceptions in 2.5 -- SystemExit, KeyboardInterrupt
+        if use_pdb:
+            import pdb
+            print >>sys.stderr, darkred('Interrupted while building, starting debugger:')
+            traceback.print_exc()
+            pdb.post_mortem(sys.exc_info()[2])
         return 1
-    except SystemExit:
-        return 0
     except Exception, err:
         if use_pdb:
             import pdb
@@ -148,7 +151,7 @@ def main(argv=sys.argv):
         else:
             if isinstance(err, SystemMessage):
                 print >>sys.stderr, darkred('reST markup error:')
-                print >>sys.stderr, str(err)
+                print >>sys.stderr, err.args[0].encode('ascii', 'backslashreplace')
             else:
                 print >>sys.stderr, darkred('Exception occurred:')
                 print >>sys.stderr, format_exception_cut_frames().rstrip()
