@@ -5,7 +5,7 @@
 
     Builder classes for different output formats.
 
-    :copyright: 2007-2008 by Georg Brandl.
+    :copyright: 2007-2008 by Georg Brandl, Sebastian Wiesner.
     :license: BSD.
 """
 
@@ -28,6 +28,7 @@ from sphinx import addnodes
 from sphinx.util import ensuredir, relative_uri, SEP, os_path
 from sphinx.htmlhelp import build_hhx
 from sphinx.htmlwriter import HTMLWriter, HTMLTranslator, SmartyPantsHTMLTranslator
+from sphinx.textwriter import TextWriter
 from sphinx.latexwriter import LaTeXWriter
 from sphinx.environment import BuildEnvironment, NoUri
 from sphinx.highlighting import PygmentsBridge
@@ -61,6 +62,9 @@ class Builder(object):
         self.info = app.info
         self.config = app.config
 
+        # images that need to be copied over (source -> dest)
+        self.images = {}
+
         # if None, this is set in load_env()
         self.env = env
         self.freshenv = freshenv
@@ -106,7 +110,7 @@ class Builder(object):
         """
         raise NotImplementedError
 
-    def status_iterator(self, iterable, summary, colorfunc):
+    def status_iterator(self, iterable, summary, colorfunc=darkgreen):
         l = -1
         for item in iterable:
             if l == -1:
@@ -117,6 +121,27 @@ class Builder(object):
         if l == 0:
             self.info()
 
+    supported_image_types = []
+
+    def post_process_images(self, doctree):
+        """
+        Pick the best candidate for all image URIs.
+        """
+        for node in doctree.traverse(nodes.image):
+            if '*' not in node['candidates']:
+                for imgtype in self.supported_image_types:
+                    candidate = node['candidates'].get(imgtype, None)
+                    if candidate:
+                        break
+                else:
+                    self.warn('%s:%s: no matching candidate for image URI %r' %
+                              (node.source, node.lineno, node['uri']))
+                    continue
+                node['uri'] = candidate
+            else:
+                candidate = node['uri']
+            self.images[candidate] = self.env.images[candidate][1]
+
     # build methods
 
     def load_env(self):
@@ -126,7 +151,7 @@ class Builder(object):
         if not self.freshenv:
             try:
                 self.info(bold('trying to load pickled env... '), nonl=True)
-                self.env = BuildEnvironment.frompickle(
+                self.env = BuildEnvironment.frompickle(self.config,
                     path.join(self.doctreedir, ENV_PICKLE_FILENAME))
                 self.info('done')
             except Exception, err:
@@ -223,7 +248,7 @@ class Builder(object):
             self.info(bold('build succeeded.'))
 
     def write(self, build_docnames, updated_docnames, method='update'):
-        if build_docnames is None:
+        if build_docnames is None or build_docnames == ['__all__']:
             # build_all
             build_docnames = self.env.found_docs
         if method == 'update':
@@ -270,6 +295,8 @@ class StandaloneHTMLBuilder(Builder):
     copysource = True
     out_suffix = '.html'
     indexer_format = 'json'
+    supported_image_types = ['image/svg+xml', 'image/png', 'image/gif',
+                             'image/jpeg']
 
     def init(self):
         """Load templates."""
@@ -302,9 +329,6 @@ class StandaloneHTMLBuilder(Builder):
     def prepare_writing(self, docnames):
         from sphinx.search import IndexBuilder
 
-        self.info(bold('creating index...'))
-        self.env.create_index(self)
-
         self.indexer = IndexBuilder()
         self.load_indexer(docnames)
         self.docwriter = HTMLWriter(self)
@@ -320,15 +344,25 @@ class StandaloneHTMLBuilder(Builder):
         else:
             self.last_updated = None
 
-        docstitle = self.config.html_title or \
-                    '%s v%s documentation' % (self.config.project,
-                                              self.config.release)
         logo = self.config.html_logo and \
                path.basename(self.config.html_logo) or ''
 
+        favicon = self.config.html_favicon and \
+                  path.basename(self.config.html_favicon) or ''
+        if favicon and os.path.splitext(favicon)[1] != '.ico':
+            self.warn('html_favicon is not an .ico file')
+
         if not isinstance(self.config.html_use_opensearch, basestring):
             self.warn('html_use_opensearch config value must now be a string')
 
+        self.relations = self.env.collect_relations()
+
+        rellinks = []
+        if self.config.html_use_index:
+            rellinks.append(('genindex', 'General Index', 'I', 'index'))
+        if self.config.html_use_modindex:
+            rellinks.append(('modindex', 'Global Module Index', 'M', 'modules'))
+
         self.globalcontext = dict(
             project = self.config.project,
             release = self.config.release,
@@ -336,92 +370,121 @@ class StandaloneHTMLBuilder(Builder):
             last_updated = self.last_updated,
             copyright = self.config.copyright,
             style = self.config.html_style,
-            use_modindex = self.config.html_use_modindex,
             use_opensearch = self.config.html_use_opensearch,
-            docstitle = docstitle,
+            docstitle = self.config.html_title,
+            shorttitle = self.config.html_short_title,
+            show_sphinx = self.config.html_show_sphinx,
+            rellinks = rellinks,
             builder = self.name,
             parents = [],
-            titles = {},
             logo = logo,
+            favicon = favicon,
             len = len, # the built-in
         )
 
-    def write_doc(self, docname, doctree):
-        destination = StringOutput(encoding='utf-8')
-        doctree.settings = self.docsettings
-
-        self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
-        self.docwriter.write(doctree, destination)
-        self.docwriter.assemble_parts()
-
+    def get_doc_context(self, docname, body):
+        """Collect items for the template context of a page."""
+        # find out relations
         prev = next = None
         parents = []
-        related = self.env.toctree_relations.get(docname)
+        rellinks = self.globalcontext['rellinks'][:]
+        related = self.relations.get(docname)
         titles = self.env.titles
-        if related:
-            try:
-                prev = {'link': self.get_relative_uri(docname, related[1]),
-                        'title': self.render_partial(titles[related[1]])['title']}
-            except KeyError:
-                # the relation is (somehow) not in the TOC tree, handle that gracefully
-                prev = None
+        if related and related[2]:
             try:
                 next = {'link': self.get_relative_uri(docname, related[2]),
                         'title': self.render_partial(titles[related[2]])['title']}
+                rellinks.append((related[2], next['title'], 'N', 'next'))
             except KeyError:
                 next = None
-        while related:
+        if related and related[1]:
+            try:
+                prev = {'link': self.get_relative_uri(docname, related[1]),
+                        'title': self.render_partial(titles[related[1]])['title']}
+                rellinks.append((related[1], prev['title'], 'P', 'previous'))
+            except KeyError:
+                # the relation is (somehow) not in the TOC tree, handle that gracefully
+                prev = None
+        while related and related[0]:
             try:
                 parents.append(
                     {'link': self.get_relative_uri(docname, related[0]),
                      'title': self.render_partial(titles[related[0]])['title']})
             except KeyError:
                 pass
-            related = self.env.toctree_relations.get(related[0])
+            related = self.relations.get(related[0])
         if parents:
             parents.pop() # remove link to the master file; we have a generic
                           # "back to index" link already
         parents.reverse()
 
+        # title rendered as HTML
         title = titles.get(docname)
-        if title:
-            title = self.render_partial(title)['title']
-        else:
-            title = ''
-        self.globalcontext['titles'][docname] = title
+        title = title and self.render_partial(title)['title'] or ''
+        # the name for the copied source
         sourcename = self.config.html_copy_source and docname + '.txt' or ''
-        ctx = dict(
-            title = title,
-            sourcename = sourcename,
-            body = self.docwriter.parts['fragment'],
-            toc = self.render_partial(self.env.get_toc_for(docname))['fragment'],
-            # only display a TOC if there's more than one item to show
-            display_toc = (self.env.toc_num_entries[docname] > 1),
+
+        # metadata for the document
+        meta = self.env.metadata.get(docname)
+
+        return dict(
             parents = parents,
             prev = prev,
             next = next,
+            title = title,
+            meta = meta,
+            body = body,
+            rellinks = rellinks,
+            sourcename = sourcename,
+            toc = self.render_partial(self.env.get_toc_for(docname))['fragment'],
+            # only display a TOC if there's more than one item to show
+            display_toc = (self.env.toc_num_entries[docname] > 1),
         )
 
-        self.index_page(docname, doctree, title)
-        self.handle_page(docname, ctx)
+    def write_doc(self, docname, doctree):
+        self.post_process_images(doctree)
+        destination = StringOutput(encoding='utf-8')
+        doctree.settings = self.docsettings
+
+        self.imgpath = relative_uri(self.get_target_uri(docname), '_images')
+        self.docwriter.write(doctree, destination)
+        self.docwriter.assemble_parts()
+        body = self.docwriter.parts['fragment']
+
+        ctx = self.get_doc_context(docname, body)
+        self.index_page(docname, doctree, ctx.get('title', ''))
+        self.handle_page(docname, ctx, event_arg=doctree)
 
     def finish(self):
         self.info(bold('writing additional files...'), nonl=1)
 
         # the global general index
 
-        # the total count of lines for each index letter, used to distribute
-        # the entries into two columns
-        indexcounts = []
-        for _, entries in self.env.index:
-            indexcounts.append(sum(1 + len(subitems) for _, (_, subitems) in entries))
+        if self.config.html_use_index:
+            # the total count of lines for each index letter, used to distribute
+            # the entries into two columns
+            genindex = self.env.create_index(self)
+            indexcounts = []
+            for _, entries in genindex:
+                indexcounts.append(sum(1 + len(subitems)
+                                       for _, (_, subitems) in entries))
 
-        genindexcontext = dict(
-            genindexentries = self.env.index,
-            genindexcounts = indexcounts,
-        )
-        self.info(' genindex', nonl=1)
-        self.handle_page('genindex', genindexcontext, 'genindex.html')
+            genindexcontext = dict(
+                genindexentries = genindex,
+                genindexcounts = indexcounts,
+                split_index = self.config.html_split_index,
+            )
+            self.info(' genindex', nonl=1)
+
+            if self.config.html_split_index:
+                self.handle_page('genindex', genindexcontext, 'genindex-split.html')
+                self.handle_page('genindex-all', genindexcontext, 'genindex.html')
+                for (key, entries), count in zip(genindex, indexcounts):
+                    ctx = {'key': key, 'entries': entries, 'count': count,
+                           'genindexentries': genindex}
+                    self.handle_page('genindex-' + key, ctx, 'genindex-single.html')
+            else:
+                self.handle_page('genindex', genindexcontext, 'genindex.html')
 
         # the global module index
 
@@ -436,35 +499,49 @@ class StandaloneHTMLBuilder(Builder):
             platforms = set()
             # sort out collapsable modules
             modindexentries = []
+            letters = []
             pmn = ''
+            num_toplevels = 0
+            num_collapsables = 0
             cg = 0 # collapse group
             fl = '' # first letter
             for mn, (fn, sy, pl, dep) in modules:
                 pl = pl and pl.split(', ') or []
                 platforms.update(pl)
                 if fl != mn[0].lower() and mn[0] != '_':
+                    # heading
                     modindexentries.append(['', False, 0, False,
                                             mn[0].upper(), '', [], False])
+                    letters.append(mn[0].upper())
                 tn = mn.split('.')[0]
                 if tn != mn:
                     # submodule
                     if pmn == tn:
                         # first submodule - make parent collapsable
                         modindexentries[-1][1] = True
+                        num_collapsables += 1
                     elif not pmn.startswith(tn):
                         # submodule without parent in list, add dummy entry
                         cg += 1
                         modindexentries.append([tn, True, cg, False, '', '', [], False])
                 else:
+                    num_toplevels += 1
                     cg += 1
                 modindexentries.append([mn, False, cg, (tn != mn), fn, sy, pl, dep])
                 pmn = mn
                 fl = mn[0].lower()
             platforms = sorted(platforms)
 
+            # apply heuristics when to collapse modindex at page load:
+            # only collapse if number of toplevel modules is larger than
+            # number of submodules
+            collapse = len(modules) - num_toplevels < num_toplevels
+
             modindexcontext = dict(
                 modindexentries = modindexentries,
                 platforms = platforms,
+                letters = letters,
+                collapse_modindex = collapse,
             )
             self.info(' modindex', nonl=1)
             self.handle_page('modindex', modindexcontext, 'modindex.html')
@@ -487,10 +564,10 @@ class StandaloneHTMLBuilder(Builder):
         self.info()
 
         # copy image files
-        if self.env.images:
+        if self.images:
             self.info(bold('copying images...'), nonl=1)
             ensuredir(path.join(self.outdir, '_images'))
-            for src, (_, dest) in self.env.images.iteritems():
+            for src, dest in self.images.iteritems():
                 self.info(' '+src, nonl=1)
                 shutil.copyfile(path.join(self.srcdir, src),
                                 path.join(self.outdir, '_images', dest))
@@ -504,9 +581,15 @@ class StandaloneHTMLBuilder(Builder):
                           for spath in self.config.html_static_path]
         for staticdirname in staticdirnames:
             for filename in os.listdir(staticdirname):
-                if not filename.startswith('.'):
-                    shutil.copyfile(path.join(staticdirname, filename),
-                                    path.join(self.outdir, '_static', filename))
+                if filename.startswith('.'):
+                    continue
+                fullname = path.join(staticdirname, filename)
+                targetname = path.join(self.outdir, '_static', filename)
+                if path.isfile(fullname):
+                    shutil.copyfile(fullname, targetname)
+                elif path.isdir(fullname):
+                    shutil.rmtree(targetname)
+                    shutil.copytree(fullname, targetname)
         # add pygments style file
         f = open(path.join(self.outdir, '_static', 'pygments.css'), 'w')
         f.write(PygmentsBridge('html', self.config.pygments_style).get_stylesheet())
@@ -561,9 +644,10 @@ class StandaloneHTMLBuilder(Builder):
         return docname + self.out_suffix
 
     def handle_page(self, pagename, addctx, templatename='page.html',
-                    outfilename=None):
+                    outfilename=None, event_arg=None):
         ctx = self.globalcontext.copy()
-        ctx['current_page_name'] = pagename
+        # current_page_name is backwards compatibility
+        ctx['pagename'] = ctx['current_page_name'] = pagename
 
         def pathto(otheruri, resource=False,
                    baseuri=self.get_target_uri(pagename)):
@@ -572,11 +656,11 @@ class StandaloneHTMLBuilder(Builder):
             return relative_uri(baseuri, otheruri)
         ctx['pathto'] = pathto
         ctx['hasdoc'] = lambda name: name in self.env.all_docs
-        sidebarfile = self.config.html_sidebars.get(pagename)
-        if sidebarfile:
-            ctx['customsidebar'] = sidebarfile
+        ctx['customsidebar'] = self.config.html_sidebars.get(pagename)
         ctx.update(addctx)
 
+        self.app.emit('html-page-context', pagename, templatename, ctx, event_arg)
+
         output = self.templates.render(templatename, ctx)
         if not outfilename:
             outfilename = path.join(self.outdir, os_path(pagename) + self.out_suffix)
@@ -612,6 +696,8 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
     name = 'pickle'
     out_suffix = '.fpickle'
     indexer_format = 'pickle'
+    supported_image_types = ('image/svg+xml', 'image/png', 'image/gif',
+                             'image/jpeg')
 
     def init(self):
         self.init_translator_class()
@@ -687,6 +773,7 @@ class HTMLHelpBuilder(StandaloneHTMLBuilder):
 
     # don't copy the reST source
     copysource = False
+    supported_image_types = ['image/png', 'image/gif', 'image/jpeg']
 
     def init(self):
         StandaloneHTMLBuilder.init(self)
@@ -702,6 +789,8 @@ class LaTeXBuilder(Builder):
     Builds LaTeX output to create PDF.
     """
     name = 'latex'
+    supported_image_types = ['application/pdf', 'image/png', 'image/gif',
+                             'image/jpeg']
 
     def init(self):
         self.docnames = []
@@ -763,6 +852,7 @@ class LaTeXBuilder(Builder):
             self.info("processing " + targetname + "... ", nonl=1)
             doctree = self.assemble_doctree(docname, toctree_only,
                 appendices=(docclass == 'manual') and appendices or [])
+            self.post_process_images(doctree)
             self.info("writing... ", nonl=1)
             doctree.settings = docsettings
             doctree.settings.author = author
@@ -828,9 +918,9 @@ class LaTeXBuilder(Builder):
 
     def finish(self):
         # copy image files
-        if self.env.images:
+        if self.images:
             self.info(bold('copying images...'), nonl=1)
-            for src, (_, dest) in self.env.images.iteritems():
+            for src, dest in self.images.iteritems():
                 self.info(' '+src, nonl=1)
                 shutil.copyfile(path.join(self.srcdir, src),
                                 path.join(self.outdir, dest))
@@ -904,17 +994,15 @@ class ChangesBuilder(Builder):
                 otherchanges.setdefault((docname, title), []).append(
                     (entry, docname, lineno))
 
-        docstitle = self.config.html_title or \
-                    '%s v%s documentation' % (self.config.project,
-                                              self.config.release)
-
         ctx = {
             'project': self.config.project,
             'version': version,
-            'docstitle': docstitle,
+            'docstitle': self.config.html_title,
+            'shorttitle': self.config.html_short_title,
             'libchanges': sorted(libchanges.iteritems()),
             'apichanges': sorted(apichanges),
             'otherchanges': sorted(otherchanges.iteritems()),
+            'show_sphinx': self.config.html_show_sphinx,
         }
         f = open(path.join(self.outdir, 'index.html'), 'w')
         try:
@@ -965,6 +1053,56 @@ class ChangesBuilder(Builder):
     def finish(self):
         pass
 
+
+class TextBuilder(Builder):
+    name = 'text'
+    out_suffix = '.txt'
+
+    def init(self):
+        pass
+
+    def get_outdated_docs(self):
+        for docname in self.env.found_docs:
+            if docname not in self.env.all_docs:
+                yield docname
+                continue
+            targetname = self.env.doc2path(docname, self.outdir, self.out_suffix)
+            try:
+                targetmtime = path.getmtime(targetname)
+            except Exception:
+                targetmtime = 0
+            try:
+                srcmtime = path.getmtime(self.env.doc2path(docname))
+                if srcmtime > targetmtime:
+                    yield docname
+            except EnvironmentError:
+                # source doesn't exist anymore
+                pass
+
+    def get_target_uri(self, docname, typ=None):
+        return ''
+
+    def prepare_writing(self, docnames):
+        self.writer = TextWriter(self)
+
+    def write_doc(self, docname, doctree):
+        destination = StringOutput(encoding='utf-8')
+        self.writer.write(doctree, destination)
+        outfilename = path.join(self.outdir, os_path(docname) + self.out_suffix)
+        ensuredir(path.dirname(outfilename)) # normally different from self.outdir
+        try:
+            f = codecs.open(outfilename, 'w', 'utf-8')
+            try:
+                f.write(self.writer.output)
+            finally:
+                f.close()
+        except (IOError, OSError), err:
+            self.warn("Error writing file %s: %s" % (outfilename, err))
+
+    def finish(self):
+        pass
+
+
 # compatibility alias
 WebHTMLBuilder = PickleHTMLBuilder
 
@@ -977,6 +1115,7 @@ builtin_builders = {
     'web': PickleHTMLBuilder,
     'htmlhelp': HTMLHelpBuilder,
     'latex': LaTeXBuilder,
+    'text': TextBuilder,
     'changes': ChangesBuilder,
     'linkcheck': CheckExternalLinksBuilder,
 }
