@@ -20,41 +20,53 @@ ws_re = re.compile(r'\s+')
 
 # ------ information units ---------------------------------------------------------
 
-def desc_index_text(desctype, currmodule, name):
+def desc_index_text(desctype, module, name):
     if desctype == 'function':
-        if not currmodule:
+        if not module:
             return '%s() (built-in function)' % name
-        return '%s() (in module %s)' % (name, currmodule)
+        return '%s() (in module %s)' % (name, module)
     elif desctype == 'data':
-        if not currmodule:
+        if not module:
             return '%s (built-in variable)' % name
-        return '%s (in module %s)' % (name, currmodule)
+        return '%s (in module %s)' % (name, module)
     elif desctype == 'class':
-        return '%s (class in %s)' % (name, currmodule)
+        return '%s (class in %s)' % (name, module)
     elif desctype == 'exception':
         return name
     elif desctype == 'method':
         try:
             clsname, methname = name.rsplit('.', 1)
         except ValueError:
-            if currmodule:
-                return '%s() (in module %s)' % (name, currmodule)
+            if module:
+                return '%s() (in module %s)' % (name, module)
             else:
                 return '%s()' % name
-        if currmodule:
-            return '%s() (%s.%s method)' % (methname, currmodule, clsname)
+        if module:
+            return '%s() (%s.%s method)' % (methname, module, clsname)
         else:
             return '%s() (%s method)' % (methname, clsname)
+    elif desctype == 'staticmethod':
+        try:
+            clsname, methname = name.rsplit('.', 1)
+        except ValueError:
+            if module:
+                return '%s() (in module %s)' % (name, module)
+            else:
+                return '%s()' % name
+        if module:
+            return '%s() (%s.%s static method)' % (methname, module, clsname)
+        else:
+            return '%s() (%s static method)' % (methname, clsname)
     elif desctype == 'attribute':
         try:
             clsname, attrname = name.rsplit('.', 1)
         except ValueError:
-            if currmodule:
-                return '%s (in module %s)' % (name, currmodule)
+            if module:
+                return '%s (in module %s)' % (name, module)
             else:
                 return name
-        if currmodule:
-            return '%s (%s.%s attribute)' % (attrname, currmodule, clsname)
+        if module:
+            return '%s (%s.%s attribute)' % (attrname, module, clsname)
         else:
             return '%s (%s attribute)' % (attrname, clsname)
     elif desctype == 'opcode':
@@ -73,16 +85,104 @@ def desc_index_text(desctype, currmodule, name):
         raise ValueError("unhandled descenv: %s" % desctype)
 
 
+# ------ make field lists (like :param foo:) in desc bodies prettier
+
+doc_fields_with_arg = {
+    'param': 'param',
+    'parameter': 'param',
+    'arg': 'param',
+    'argument': 'param',
+    'keyword': 'param',
+    'kwarg': 'param',
+    'kwparam': 'param',
+    'type': 'type',
+    'raises': 'Raises',
+    'raise': 'Raises',
+    'exception': 'Raises',
+    'except': 'Raises',
+    'var': 'Variable',
+    'ivar': 'Variable',
+    'cvar': 'Variable',
+    'returns': 'Returns',
+    'return': 'Returns',
+}
+
+doc_fields_without_arg = {
+    'returns': 'Returns',
+    'return': 'Returns',
+    'rtype': 'Return type',
+}
+
+def handle_doc_fields(node):
+    # don't traverse, only handle field lists that are immediate children
+    for child in node.children:
+        if not isinstance(child, nodes.field_list):
+            continue
+        params = None
+        param_nodes = {}
+        param_types = {}
+        new_list = nodes.field_list()
+        for field in child:
+            fname, fbody = field
+            try:
+                typ, obj = fname.astext().split(None, 1)
+                typ = doc_fields_with_arg[typ]
+                if len(fbody.children) == 1 and \
+                   isinstance(fbody.children[0], nodes.paragraph):
+                    children = fbody.children[0].children
+                else:
+                    children = fbody.children
+                if typ == 'param':
+                    if not params:
+                        pfield = nodes.field()
+                        pfield += nodes.field_name('Parameters', 'Parameters')
+                        pfield += nodes.field_body()
+                        params = nodes.bullet_list()
+                        pfield[1] += params
+                        new_list += pfield
+                    dlitem = nodes.list_item()
+                    dlpar = nodes.paragraph()
+                    dlpar += nodes.emphasis(obj, obj)
+                    dlpar += nodes.Text(' -- ', ' -- ')
+                    dlpar += children
+                    param_nodes[obj] = dlpar
+                    dlitem += dlpar
+                    params += dlitem
+                elif typ == 'type':
+                    param_types[obj] = fbody.astext()
+                else:
+                    nfield = nodes.field()
+                    nfield += nodes.field_name(typ, typ)
+                    nfield += nodes.field_body()
+                    nfield[1] += fbody.children
+                    new_list += nfield
+            except (KeyError, ValueError):
+                fnametext = fname.astext()
+                try:
+                    typ = doc_fields_without_arg[fnametext]
+                except KeyError:
+                    # at least capitalize the field name
+                    typ = fnametext.capitalize()
+                fname[0] = nodes.Text(typ)
+                new_list += field
+        for param, type in param_types.iteritems():
+            if param in param_nodes:
+                param_nodes[param].insert(1, nodes.Text(' (%s)' % type))
+        child.replace_self(new_list)
+
+
 # ------ functions to parse a Python or C signature and create desc_* nodes.
 
-py_sig_re = re.compile(r'''^([\w.]*\.)?        # class names
-                           (\w+)  \s*          # thing name
-                           (?: \((.*)\) )? $   # optionally arguments
-                        ''', re.VERBOSE)
+py_sig_re = re.compile(
+    r'''^ ([\w.]*\.)?            # class name(s)
+          (\w+)  \s*             # thing name
+          (?: \((.*)\)           # optional arguments
+          (\s* -> \s* .*)? )? $  # optional return annotation
+          ''', re.VERBOSE)
 
 py_paramlist_re = re.compile(r'([\[\],])')  # split at '[', ']' and ','
 
-def parse_py_signature(signode, sig, desctype, env):
+def parse_py_signature(signode, sig, desctype, module, env):
     """
     Transform a python signature into RST nodes.
     Return (fully qualified name of the thing, classname if any).
@@ -94,37 +194,40 @@ def parse_py_signature(signode, sig, desctype, env):
     m = py_sig_re.match(sig)
     if m is None:
         raise ValueError
-    classname, name, arglist = m.groups()
+    classname, name, arglist, retann = m.groups()
 
-    add_module = True
     if env.currclass:
+        add_module = False
         if classname and classname.startswith(env.currclass):
             fullname = classname + name
             # class name is given again in the signature
             classname = classname[len(env.currclass):].lstrip('.')
-            add_module = False
         elif classname:
             # class name is given in the signature, but different
+            # (shouldn't happen)
             fullname = env.currclass + '.' + classname + name
         else:
             # class name is not given in the signature
             fullname = env.currclass + '.' + name
-            add_module = False
     else:
+        add_module = True
         fullname = classname and classname + name or name
 
+    if desctype == 'staticmethod':
+        signode += addnodes.desc_annotation('static ', 'static ')
+
     if classname:
-        signode += addnodes.desc_classname(classname, classname)
+        signode += addnodes.desc_addname(classname, classname)
     # exceptions are a special case, since they are documented in the
     # 'exceptions' module.
     elif add_module and env.config.add_module_names and \
-           env.currmodule and env.currmodule != 'exceptions':
-        nodetext = env.currmodule + '.'
-        signode += addnodes.desc_classname(nodetext, nodetext)
+           module and module != 'exceptions':
+        nodetext = module + '.'
+        signode += addnodes.desc_addname(nodetext, nodetext)
 
     signode += addnodes.desc_name(name, name)
     if not arglist:
-        if desctype in ('function', 'method'):
+        if desctype in ('function', 'method', 'staticmethod'):
             # for callables, add an empty parameter list
             signode += addnodes.desc_parameterlist()
         return fullname, classname
@@ -148,6 +251,9 @@ def parse_py_signature(signode, sig, desctype, env):
             stack[-1] += addnodes.desc_parameter(token, token)
     if len(stack) != 1:
         raise ValueError
+    if retann:
+        retann = u' \N{RIGHTWARDS ARROW} ' + retann.strip()[2:]
+        signode += addnodes.desc_type(retann, retann)
     return fullname, classname
 
 
@@ -199,7 +305,7 @@ def parse_c_signature(signode, sig, desctype):
     try:
         classname, funcname = name.split('::', 1)
         classname += '::'
-        signode += addnodes.desc_classname(classname, classname)
+        signode += addnodes.desc_addname(classname, classname)
         signode += addnodes.desc_name(funcname, funcname)
         # name (the full name) is still both parts
     except ValueError:
@@ -232,7 +338,7 @@ def parse_c_signature(signode, sig, desctype):
         paramlist += param
     signode += paramlist
     if const:
-        signode += addnodes.desc_classname(const, const)
+        signode += addnodes.desc_addname(const, const)
     return name
 
 
@@ -261,9 +367,9 @@ def parse_option_desc(signode, sig):
     for m in option_desc_re.finditer(sig):
         prefix, optname, args = m.groups()
         if count:
-            signode += addnodes.desc_classname(', ', ', ')
+            signode += addnodes.desc_addname(', ', ', ')
         signode += addnodes.desc_name(prefix+optname, prefix+optname)
-        signode += addnodes.desc_classname(args, args)
+        signode += addnodes.desc_addname(args, args)
         if not count:
             firstname = optname
         count += 1
@@ -284,6 +390,7 @@ def desc_directive(desctype, arguments, options, content, lineno,
     signatures = map(lambda s: s.strip().replace('\\', ''), arguments[0].split('\n'))
     names = []
     clsname = None
+    module = options.get('module', env.currmodule)
     for i, sig in enumerate(signatures):
         # add a signature node for each signature in the current unit
         # and add a reference target for it
@@ -293,8 +400,8 @@ def desc_directive(desctype, arguments, options, content, lineno,
         node.append(signode)
         try:
             if desctype in ('function', 'data', 'class', 'exception',
-                            'method', 'attribute'):
-                name, clsname = parse_py_signature(signode, sig, desctype, env)
+                            'method', 'staticmethod', 'attribute'):
+                name, clsname = parse_py_signature(signode, sig, desctype, module, env)
             elif desctype in ('cfunction', 'cmember', 'cmacro', 'ctype', 'cvar'):
                 name = parse_c_signature(signode, sig, desctype)
             elif desctype == 'opcode':
@@ -347,7 +454,7 @@ def desc_directive(desctype, arguments, options, content, lineno,
         # only add target and index entry if this is the first description of the
         # function name in this desc block
         if not noindex and name not in names:
-            fullname = (env.currmodule and env.currmodule + '.' or '') + name
+            fullname = (module and module + '.' or '') + name
             # note target
             if fullname not in state.document.ids:
                 signode['names'].append(fullname)
@@ -358,7 +465,7 @@ def desc_directive(desctype, arguments, options, content, lineno,
             names.append(name)
 
             env.note_index_entry('single',
-                                 desc_index_text(desctype, env.currmodule, name),
+                                 desc_index_text(desctype, module, name),
                                  fullname, fullname)
 
     subnode = addnodes.desc_content()
@@ -367,13 +474,15 @@ def desc_directive(desctype, arguments, options, content, lineno,
     if desctype in ('class', 'exception') and names:
         env.currclass = names[0]
         clsname_set = True
-    elif desctype in ('method', 'attribute') and clsname and not env.currclass:
+    elif desctype in ('method', 'staticmethod', 'attribute') and \
+             clsname and not env.currclass:
         env.currclass = clsname.strip('.')
         clsname_set = True
     # needed for association of version{added,changed} directives
     if names:
         env.currdesc = names[0]
     state.nested_parse(content, content_offset, subnode)
+    handle_doc_fields(subnode)
     if clsname_set:
         env.currclass = None
     env.currdesc = None
@@ -382,7 +491,8 @@ def desc_directive(desctype, arguments, options, content, lineno,
 
 desc_directive.content = 1
 desc_directive.arguments = (1, 0, 1)
-desc_directive.options = {'noindex': directives.flag}
+desc_directive.options = {'noindex': directives.flag,
+                          'module': directives.unchanged}
 
 desctypes = [
     # the Python ones
@@ -390,6 +500,7 @@ desctypes = [
     'data',
     'class',
     'method',
+    'staticmethod',
     'attribute',
     'exception',
     # the C ones
@@ -443,3 +554,6 @@ def target_directive(targettype, arguments, options, content, lineno,
 
 target_directive.content = 0
 target_directive.arguments = (1, 0, 1)
+
+# note, the target directive is not registered here, it is used by the application
+# when registering additional xref types
