@@ -14,15 +14,17 @@ import os
 import time
 import heapq
 import types
+import imghdr
 import difflib
-import itertools
 import cPickle as pickle
 from os import path
+from glob import glob
 from string import uppercase
+from itertools import izip, groupby
 try:
     import hashlib
     md5 = hashlib.md5
-except:
+except ImportError:
     # 2.4 compatibility
     import md5
     md5 = md5.new
@@ -32,6 +34,8 @@ from docutils.io import FileInput
 from docutils.core import publish_doctree
 from docutils.utils import Reporter
 from docutils.readers import standalone
+from docutils.parsers.rst import roles
+from docutils.parsers.rst.languages import en as english
 from docutils.transforms import Transform
 from docutils.transforms.parts import ContentsFilter
 from docutils.transforms.universal import FilterMessages
@@ -51,6 +55,7 @@ default_settings = {
     'embed_stylesheet': False,
     'cloak_email_addresses': True,
     'pep_base_url': 'http://www.python.org/dev/peps/',
+    'rfc_base_url': 'http://rfc.net/',
     'input_encoding': 'utf-8',
     'doctitle_xform': False,
     'sectsubtitle_xform': False,
@@ -58,7 +63,7 @@ default_settings = {
 
 # This is increased every time an environment attribute is added
 # or changed to properly invalidate pickle files.
-ENV_VERSION = 21
+ENV_VERSION = 24
 
 
 default_substitutions = set([
@@ -67,6 +72,8 @@ default_substitutions = set([
     'today',
 ])
 
+dummy_reporter = Reporter('', 4, 4)
+
 
 class RedirStream(object):
     def __init__(self, writefunc):
@@ -162,12 +169,13 @@ class BuildEnvironment:
     # --------- ENVIRONMENT PERSISTENCE ----------------------------------------
 
     @staticmethod
-    def frompickle(filename):
+    def frompickle(config, filename):
         picklefile = open(filename, 'rb')
         try:
             env = pickle.load(picklefile)
         finally:
             picklefile.close()
+        env.config.values = config.values
         if env.version != ENV_VERSION:
             raise IOError('env version not current')
         return env
@@ -176,6 +184,8 @@ class BuildEnvironment:
         # remove unpicklable attributes
         warnfunc = self._warnfunc
         self.set_warnfunc(None)
+        values = self.config.values
+        del self.config.values
         picklefile = open(filename, 'wb')
         # remove potentially pickling-problematic values from config
         for key, val in vars(self.config).items():
@@ -188,7 +198,8 @@ class BuildEnvironment:
             pickle.dump(self, picklefile, pickle.HIGHEST_PROTOCOL)
         finally:
             picklefile.close()
-        # reset stream
+        # reset attributes
+        self.config.values = values
         self.set_warnfunc(warnfunc)
 
     # --------- ENVIRONMENT INITIALIZATION -------------------------------------
@@ -198,6 +209,9 @@ class BuildEnvironment:
         self.srcdir = srcdir
         self.config = config
 
+        # the application object; only set while update() runs
+        self.app = None
+
         # the docutils settings for building
         self.settings = default_settings.copy()
         self.settings['env'] = self
@@ -225,8 +239,7 @@ class BuildEnvironment:
         self.toc_num_entries = {}   # docname -> number of real entries
                                     # used to determine when to show the TOC in a sidebar
                                     # (don't show if it's only one item)
-        self.toctree_relations = {} # docname -> ["parent", "previous", "next"] docname
-                                    # for navigating in the toctree
+        self.toctree_includes = {}  # docname -> list of toctree includefiles
         self.files_to_rebuild = {}  # docname -> set of files (containing its TOCs)
                                     # to rebuild too
 
@@ -280,11 +293,14 @@ class BuildEnvironment:
             self.titles.pop(docname, None)
             self.tocs.pop(docname, None)
             self.toc_num_entries.pop(docname, None)
+            self.toctree_includes.pop(docname, None)
             self.filemodules.pop(docname, None)
             self.indexentries.pop(docname, None)
 
-            for subfn, fnset in self.files_to_rebuild.iteritems():
+            for subfn, fnset in self.files_to_rebuild.items():
                 fnset.discard(docname)
+                if not fnset:
+                    del self.files_to_rebuild[subfn]
             for fullname, (fn, _) in self.descrefs.items():
                 if fn == docname:
                     del self.descrefs[fullname]
@@ -325,9 +341,12 @@ class BuildEnvironment:
         """
         Find all source files in the source dir and put them in self.found_docs.
         """
-        self.found_docs = set(get_matching_docs(self.srcdir, config.source_suffix,
-                                                exclude=set(config.unused_docs),
-                                                prune=['_sources']))
+        exclude_dirs  = [d.replace(SEP, path.sep) for d in config.exclude_dirs]
+        exclude_trees = [d.replace(SEP, path.sep) for d in config.exclude_trees]
+        self.found_docs = set(get_matching_docs(
+            self.srcdir, config.source_suffix, exclude_docs=set(config.unused_docs),
+            exclude_dirs=exclude_dirs, exclude_trees=exclude_trees,
+            exclude_dirnames=['_sources']))
 
     def get_outdated_files(self, config_changed):
         """
@@ -391,9 +410,7 @@ class BuildEnvironment:
             for key, descr in config.config_values.iteritems():
                 if not descr[1]:
                     continue
-                if not hasattr(self.config, key) or \
-                   self.config[key] != config[key]:
-
+                if self.config[key] != config[key]:
                     msg = '[config changed] '
                     config_changed = True
                     break
@@ -406,6 +423,7 @@ class BuildEnvironment:
         yield msg
 
         self.config = config
+        self.app = app
 
         # clear all files no longer present
         for docname in removed:
@@ -420,6 +438,8 @@ class BuildEnvironment:
             self.warn(None, 'master file %s not found' %
                       self.doc2path(config.master_doc))
 
+        self.app = None
+
         # remove all non-existing images from inventory
         for imgsrc in self.images.keys():
             if not os.access(path.join(self.srcdir, imgsrc), os.R_OK):
@@ -439,6 +459,14 @@ class BuildEnvironment:
         if src_path is None:
             src_path = self.doc2path(docname)
 
+        if self.config.default_role:
+            role_fn, messages = roles.role(self.config.default_role, english,
+                                           0, dummy_reporter)
+            if role_fn:
+                roles._roles[''] = role_fn
+            else:
+                self.warn(docname, 'default role %s not found' %
+                          self.config.default_role)
         self.docname = docname
         doctree = publish_doctree(None, src_path, FileInput,
                                   settings_overrides=self.settings,
@@ -503,25 +531,48 @@ class BuildEnvironment:
         existing_names = set(v[1] for v in self.images.itervalues())
         docdir = path.dirname(self.doc2path(docname, base=None))
         for node in doctree.traverse(nodes.image):
+            # Map the mimetype to the corresponding image.  The writer may
+            # choose the best image from these candidates.  The special key * is
+            # set if there is only single candiate to be used by a writer.
+            node['candidates'] = candidates = {}
             imguri = node['uri']
             if imguri.find('://') != -1:
                 self.warn(docname, 'Nonlocal image URI found: %s' % imguri, node.line)
+                candidates['*'] = imguri
+                continue
+            imgpath = path.normpath(path.join(docdir, imguri))
+            node['uri'] = imgpath
+            if imgpath.endswith(os.extsep + '*'):
+                for filename in glob(imgpath):
+                    basename, ext = os.path.splitext(filename)
+                    if ext == '.pdf':
+                        candidates['application/pdf'] = filename
+                    elif ext == '.svg':
+                        candidates['image/svg+xml'] = filename
+                    else:
+                        f = open(filename, 'rb')
+                        try:
+                            imgtype = imghdr.what(f)
+                        finally:
+                            f.close()
+                        if imgtype:
+                            candidates['image/' + imgtype] = filename
             else:
-                imgpath = path.normpath(path.join(docdir, imguri))
-                node['uri'] = imgpath
-                self.dependencies.setdefault(docname, set()).add(imgpath)
-                if not os.access(path.join(self.srcdir, imgpath), os.R_OK):
-                    self.warn(docname, 'Image file not readable: %s' % imguri, node.line)
-                if imgpath in self.images:
-                    self.images[imgpath][0].add(docname)
+                candidates['*'] = imgpath
+            for img in candidates.itervalues():
+                self.dependencies.setdefault(docname, set()).add(img)
+                if not os.access(path.join(self.srcdir, img), os.R_OK):
+                    self.warn(docname, 'Image file not readable: %s' % img, node.line)
+                if img in self.images:
+                    self.images[img][0].add(docname)
                     continue
-                uniquename = path.basename(imgpath)
+                uniquename = path.basename(img)
                 base, ext = path.splitext(uniquename)
                 i = 0
                 while uniquename in existing_names:
                     i += 1
                     uniquename = '%s%s%s' % (base, i, ext)
-                self.images[imgpath] = (set([docname]), uniquename)
+                self.images[img] = (set([docname]), uniquename)
                 existing_names.add(uniquename)
 
     def process_metadata(self, docname, doctree):
@@ -587,23 +638,22 @@ class BuildEnvironment:
         """Note a TOC tree directive in a document and gather information about
            file relations from it."""
         includefiles = toctreenode['includefiles']
-        includefiles_len = len(includefiles)
-        for i, includefile in enumerate(includefiles):
-            # the "previous" file for the first toctree item is the parent
-            previous = i > 0 and includefiles[i-1] or docname
-            # the "next" file for the last toctree item is the parent again
-            next = i < includefiles_len-1 and includefiles[i+1] or docname
-            self.toctree_relations[includefile] = [docname, previous, next]
+        for includefile in includefiles:
             # note that if the included file is rebuilt, this one must be
             # too (since the TOC of the included file could have changed)
             self.files_to_rebuild.setdefault(includefile, set()).add(docname)
-
+        self.toctree_includes.setdefault(docname, []).extend(includefiles)
 
     def build_toc_from(self, docname, document):
         """Build a TOC from the doctree and store it in the inventory."""
         numentries = [0] # nonlocal again...
 
-        def build_toc(node):
+        try:
+            maxdepth = int(self.metadata[docname].get('tocdepth', 0))
+        except ValueError:
+            maxdepth = 0
+
+        def build_toc(node, depth=1):
             entries = []
             for subnode in node:
                 if isinstance(subnode, addnodes.toctree):
@@ -634,7 +684,8 @@ class BuildEnvironment:
                                             *nodetext)
                 para = addnodes.compact_paragraph('', '', reference)
                 item = nodes.list_item('', para)
-                item += build_toc(subnode)
+                if maxdepth == 0 or depth < maxdepth:
+                    item += build_toc(subnode, depth+1)
                 entries.append(item)
             if entries:
                 return nodes.bullet_list('', *entries)
@@ -696,11 +747,14 @@ class BuildEnvironment:
             doctree = pickle.load(f)
         finally:
             f.close()
+        doctree.settings.env = self
         doctree.reporter = Reporter(self.doc2path(docname), 2, 4,
                                     stream=RedirStream(self._warnfunc))
         return doctree
 
-    def get_and_resolve_doctree(self, docname, builder, doctree=None):
+
+    def get_and_resolve_doctree(self, docname, builder, doctree=None,
+                                prune_toctrees=True):
         """Read the doctree from the pickle, resolve cross-references and
            toctrees and return it."""
         if doctree is None:
@@ -710,6 +764,40 @@ class BuildEnvironment:
         self.resolve_references(doctree, docname, builder)
 
         # now, resolve all toctree nodes
+        for toctreenode in doctree.traverse(addnodes.toctree):
+            result = self.resolve_toctree(docname, builder, toctreenode,
+                                          prune=prune_toctrees)
+            if result is None:
+                toctreenode.replace_self([])
+            else:
+                toctreenode.replace_self(result)
+
+        return doctree
+
+    def resolve_toctree(self, docname, builder, toctree, prune=True, maxdepth=0,
+                        titles_only=False):
+        """
+        Resolve a *toctree* node into individual bullet lists with titles
+        as items, returning None (if no containing titles are found) or
+        a new node.
+
+        If *prune* is True, the tree is pruned to *maxdepth*, or if that is 0,
+        to the value of the *maxdepth* option on the *toctree* node.
+        If *titles_only* is True, only toplevel document titles will be in the
+        resulting tree.
+        """
+
+        def _walk_depth(node, depth, maxdepth, titleoverrides):
+            """Utility: Cut a TOC at a specified depth."""
+            for subnode in node.children[:]:
+                if isinstance(subnode, (addnodes.compact_paragraph, nodes.list_item)):
+                    _walk_depth(subnode, depth, maxdepth, titleoverrides)
+                elif isinstance(subnode, nodes.bullet_list):
+                    if depth > maxdepth:
+                        subnode.parent.replace(subnode, [])
+                    else:
+                        _walk_depth(subnode, depth+1, maxdepth, titleoverrides)
+
         def _entries_from_toctree(toctreenode, separate=False):
             """Return TOC entries for a toctree node."""
             includefiles = map(str, toctreenode['includefiles'])
@@ -723,6 +811,15 @@ class BuildEnvironment:
                     self.warn(docname, 'toctree contains ref to nonexisting '
                               'file %r' % includefile)
                 else:
+                    # if titles_only is given, only keep the main title and
+                    # sub-toctrees
+                    if titles_only:
+                        # delete everything but the toplevel title(s) and toctrees
+                        for toplevel in toc:
+                            # nodes with length 1 don't have any children anyway
+                            if len(toplevel) > 1:
+                                subtoctrees = toplevel.traverse(addnodes.toctree)
+                                toplevel[1][:] = subtoctrees
                     # resolve all sub-toctrees
                     for toctreenode in toc.traverse(addnodes.toctree):
                         i = toctreenode.parent.index(toctreenode) + 1
@@ -736,51 +833,31 @@ class BuildEnvironment:
                         entries.extend(toc.children)
             return entries
 
-        def _walk_depth(node, depth, maxdepth, titleoverrides):
-            """Utility: Cut a TOC at a specified depth."""
-            for subnode in node.children[:]:
-                if isinstance(subnode, (addnodes.compact_paragraph, nodes.list_item)):
-                    _walk_depth(subnode, depth, maxdepth, titleoverrides)
-                elif isinstance(subnode, nodes.bullet_list):
-                    if depth > maxdepth:
-                        subnode.parent.replace(subnode, [])
-                    else:
-                        _walk_depth(subnode, depth+1, maxdepth, titleoverrides)
+        maxdepth = maxdepth or toctree.get('maxdepth', -1)
+        titleoverrides = toctree.get('includetitles', {})
 
-        for toctreenode in doctree.traverse(addnodes.toctree):
-            maxdepth = toctreenode.get('maxdepth', -1)
-            titleoverrides = toctreenode.get('includetitles', {})
-            tocentries = _entries_from_toctree(toctreenode, separate=True)
-            if tocentries:
-                newnode = addnodes.compact_paragraph('', '', *tocentries)
-                # prune the tree to maxdepth and replace titles
-                if maxdepth > 0:
-                    _walk_depth(newnode, 1, maxdepth, titleoverrides)
-                # replace titles, if needed
-                if titleoverrides:
-                    for refnode in newnode.traverse(nodes.reference):
-                        if refnode.get('anchorname', None):
-                            continue
-                        if refnode['refuri'] in titleoverrides:
-                            newtitle = titleoverrides[refnode['refuri']]
-                            refnode.children = [nodes.Text(newtitle)]
-                toctreenode.replace_self(newnode)
-            else:
-                toctreenode.replace_self([])
-
-        # set the target paths in the toctrees (they are not known
-        # at TOC generation time)
-        for node in doctree.traverse(nodes.reference):
-            if node.hasattr('anchorname'):
-                # a TOC reference
-                node['refuri'] = builder.get_relative_uri(
-                    docname, node['refuri']) + node['anchorname']
-
-        return doctree
+        tocentries = _entries_from_toctree(toctree, separate=True)
+        if not tocentries:
+            return None
 
+        newnode = addnodes.compact_paragraph('', '', *tocentries)
+        newnode['toctree'] = True
+        # prune the tree to maxdepth and replace titles
+        if maxdepth > 0 and prune:
+            _walk_depth(newnode, 1, maxdepth, titleoverrides)
+        # replace titles, if needed, and set the target paths in the
+        # toctrees (they are not known at TOC generation time)
+        for refnode in newnode.traverse(nodes.reference):
+            refnode['refuri'] = builder.get_relative_uri(
+                docname, refnode['refuri']) + refnode['anchorname']
+            if titleoverrides and not refnode['anchorname'] \
+                   and refnode['refuri'] in titleoverrides:
+                newtitle = titleoverrides[refnode['refuri']]
+                refnode.children = [nodes.Text(newtitle)]
+        return newnode
 
     descroles = frozenset(('data', 'exc', 'func', 'class', 'const', 'attr',
-                           'meth', 'cfunc', 'cdata', 'ctype', 'cmacro'))
+                           'meth', 'cfunc', 'cdata', 'ctype', 'cmacro', 'obj'))
 
     def resolve_references(self, doctree, fromdocname, builder):
         for node in doctree.traverse(addnodes.pending_xref):
@@ -930,7 +1007,7 @@ class BuildEnvironment:
                 if type == 'single':
                     try:
                         entry, subentry = string.split(';', 1)
-                    except:
+                    except ValueError:
                         entry, subentry = string, ''
                     add_entry(entry.strip(), subentry.strip())
                 elif type == 'pair':
@@ -993,14 +1070,57 @@ class BuildEnvironment:
             else:
                 # get all other symbols under one heading
                 return 'Symbols'
-        self.index = [(key, list(group)) for (key, group) in
-                      itertools.groupby(newlist, keyfunc)]
+        return [(key, list(group)) for (key, group) in groupby(newlist, keyfunc)]
+
+    def collect_relations(self):
+        relations = {}
+        getinc = self.toctree_includes.get
+        def collect(parents, docname, previous, next):
+            includes = getinc(docname)
+            # previous
+            if not previous:
+                # if no previous sibling, go to parent
+                previous = parents[0][0]
+            else:
+                # else, go to previous sibling, or if it has children, to
+                # the last of its children, or if that has children, to the
+                # last of those, and so forth
+                while 1:
+                    previncs = getinc(previous)
+                    if previncs:
+                        previous = previncs[-1]
+                    else:
+                        break
+            # next
+            if includes:
+                # if it has children, go to first of them
+                next = includes[0]
+            elif next:
+                # else, if next sibling, go to it
+                pass
+            else:
+                # else, go to the next sibling of the parent, if present,
+                # else the grandparent's sibling, if present, and so forth
+                for parname, parindex in parents:
+                    parincs = getinc(parname)
+                    if parincs and parindex + 1 < len(parincs):
+                        next = parincs[parindex+1]
+                        break
+                # else it will stay None
+            # same for children
+            if includes:
+                for subindex, args in enumerate(izip(includes, [None] + includes,
+                                                     includes[1:] + [None])):
+                    collect([(docname, subindex)] + parents, *args)
+            relations[docname] = [parents[0][0], previous, next]
+        collect([(None, 0)], self.config.master_doc, None, None)
+        return relations
 
     def check_consistency(self):
         """Do consistency checks."""
 
-        for docname in self.all_docs:
-            if docname not in self.toctree_relations:
+        for docname in sorted(self.all_docs):
+            if docname not in self.files_to_rebuild:
                 if docname == self.config.master_doc:
                     # the master file is not included anywhere ;)
                     continue
