@@ -12,6 +12,7 @@
 import sys, os, time
 from os import path
 
+from sphinx.util import make_filename
 from sphinx.util.console import purple, bold, red, nocolor
 
 
@@ -77,6 +78,9 @@ today_fmt = '%%B %%d, %%Y'
 # for source files.
 #exclude_dirs = []
 
+# The reST default role (used for this markup: `text`) to use for all documents.
+#default_role = None
+
 # If true, '()' will be appended to :func: etc. cross-reference text.
 #add_function_parentheses = True
 
@@ -104,10 +108,18 @@ html_style = 'default.css'
 # "<project> v<release> documentation".
 #html_title = None
 
+# A shorter title for the navigation bar.  Default is the same as html_title.
+#html_short_title = None
+
 # The name of an image file (within the static path) to place at the top of
 # the sidebar.
 #html_logo = None
 
+# The name of an image file (within the static path) to use as favicon of the
+# docs.  This file should be a Windows icon file (.ico) being 16x16 or 32x32
+# pixels large.
+#html_favicon = None
+
 # Add any paths that contain custom static files (such as style sheets) here,
 # relative to this directory. They are copied after the builtin static files,
 # so a file named "default.css" will overwrite the builtin "default.css".
@@ -131,6 +143,12 @@ html_last_updated_fmt = '%%b %%d, %%Y'
 # If false, no module index is generated.
 #html_use_modindex = True
 
+# If false, no index is generated.
+#html_use_index = True
+
+# If true, the index is split into individual pages for each letter.
+#html_split_index = False
+
 # If true, the reST sources are included in the HTML build as _sources/<name>.
 #html_copy_source = True
 
@@ -143,7 +161,7 @@ html_last_updated_fmt = '%%b %%d, %%Y'
 #html_file_suffix = ''
 
 # Output file base name for HTML help builder.
-htmlhelp_basename = '%(project)sdoc'
+htmlhelp_basename = '%(project_fn)sdoc'
 
 
 # Options for LaTeX output
@@ -158,7 +176,7 @@ htmlhelp_basename = '%(project)sdoc'
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
 latex_documents = [
-  ('%(master)s', '%(project)s.tex', '%(project)s Documentation', '%(author)s', 'manual'),
+  ('%(master)s', '%(project_fn)s.tex', '%(project)s Documentation', '%(author)s', 'manual'),
 ]
 
 # The name of an image file (relative to this directory) to place at the top of
@@ -380,6 +398,7 @@ directly.'''
     do_prompt(d, 'makefile', 'Create Makefile? (y/n)',
               os.name == 'posix' and 'y' or 'n', boolean)
 
+    d['project_fn'] = make_filename(d['project'])
     d['year'] = time.strftime('%Y')
     d['now'] = time.asctime()
     d['underline'] = len(d['project']) * '='
