@@ -39,6 +39,8 @@ for rolename, nodeclass in generic_docroles.iteritems():
 
 def indexmarkup_role(typ, rawtext, etext, lineno, inliner, options={}, content=[]):
     env = inliner.document.settings.env
+    if not typ:
+        typ = env.config.default_role
     text = utils.unescape(etext)
     targetid = 'index-%s' % env.index_num
     env.index_num += 1
@@ -102,9 +104,7 @@ innernodetypes = {
     'option': addnodes.literal_emphasis,
 }
 
-def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
-    env = inliner.document.settings.env
-    text = utils.unescape(text)
+def _fix_parens(typ, text, env):
     if typ in ('func', 'meth', 'cfunc'):
         if text.endswith('()'):
             # remove parentheses
@@ -112,9 +112,16 @@ def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
         if env.config.add_function_parentheses:
             # add them back to all occurrences if configured
             text += '()'
+    return text
+
+def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
+    env = inliner.document.settings.env
+    if not typ:
+        typ = env.config.default_role
+    text = utils.unescape(text)
     # if the first character is a bang, don't cross-reference at all
     if text[0:1] == '!':
-        text = text[1:]
+        text = _fix_parens(typ, text[1:], env)
         return [innernodetypes.get(typ, nodes.literal)(
             rawtext, text, classes=['xref'])], []
     # we want a cross-reference, create the reference node
@@ -122,48 +129,57 @@ def xfileref_role(typ, rawtext, text, lineno, inliner, options={}, content=[]):
                                   modname=env.currmodule, classname=env.currclass)
     # we may need the line number for warnings
     pnode.line = lineno
-    innertext = text
-    # special actions for Python object cross-references
-    if typ in ('data', 'exc', 'func', 'class', 'const', 'attr', 'meth', 'mod'):
-        # if the first character is a dot, search more specific namespaces first
-        # else search builtins first
-        if text[0:1] == '.':
-            text = text[1:]
-            pnode['refspecific'] = True
-        # if the first character is a tilde, don't display the module/class parts
-        # of the contents
-        elif text[0:1] == '~':
-            text = text[1:]
-            dot = text.rfind('.')
-            if dot != -1:
-                innertext = text[dot+1:]
-    # look if explicit title and target are given
+    # the link title may differ from the target, but by default they are the same
+    title = target = text
+    titleistarget = True
+    # look if explicit title and target are given with `foo <bar>` syntax
     brace = text.find('<')
     if brace != -1:
+        titleistarget = False
         pnode['refcaption'] = True
         m = caption_ref_re.match(text)
         if m:
             target = m.group(2)
-            innertext = m.group(1)
+            title = m.group(1)
         else:
             # fallback: everything after '<' is the target
             target = text[brace+1:]
-            innertext = text[:brace]
-    # else, generate target from title
+            title = text[:brace]
+    # special target  for Python object cross-references
+    if typ in ('data', 'exc', 'func', 'class', 'const', 'attr', 'meth', 'mod', 'obj'):
+        # fix-up parentheses in link title
+        if titleistarget:
+            title = title.lstrip('.')   # only has a meaning for the target
+            target = target.lstrip('~') # only has a meaning for the title
+            title = _fix_parens(typ, title, env)
+            # if the first character is a tilde, don't display the module/class
+            # parts of the contents
+            if title[0:1] == '~':
+                title = title[1:]
+                dot = title.rfind('.')
+                if dot != -1:
+                    title = title[dot+1:]
+        # remove parentheses from the target too
+        if target.endswith('()'):
+            target = target[:-2]
+        # if the first character is a dot, search more specific namespaces first
+        # else search builtins first
+        if target[0:1] == '.':
+            target = target[1:]
+            pnode['refspecific'] = True
+    # some other special cases for the target
+    elif typ == 'option' and target[0] in '-/':
+        # strip option marker from target
+        target = target[1:]
     elif typ == 'term':
         # normalize whitespace in definition terms (if the term reference is
-        # broken over a line, a newline will be in text)
-        target = ws_re.sub(' ', text).lower()
-    elif typ == 'option':
-        # strip option marker from target
-        if text[0] in '-/':
-            target = text[1:]
-        else:
-            target = text
+        # broken over a line, a newline will be in target)
+        target = ws_re.sub(' ', target).lower()
     else:
-        target = ws_re.sub('', text)
+        # remove all whitespace to avoid referencing problems
+        target = ws_re.sub('', target)
     pnode['reftarget'] = target
-    pnode += innernodetypes.get(typ, nodes.literal)(rawtext, innertext, classes=['xref'])
+    pnode += innernodetypes.get(typ, nodes.literal)(rawtext, title, classes=['xref'])
     return [pnode], []
 
 
@@ -197,7 +213,7 @@ specific_docroles = {
     'const': xfileref_role,
     'attr': xfileref_role,
     'meth': xfileref_role,
-
+    'obj': xfileref_role,
     'cfunc' : xfileref_role,
     'cdata' : xfileref_role,
     'ctype' : xfileref_role,
