@@ -10,6 +10,7 @@
 """
 
 import os
+import re
 import sys
 import fnmatch
 import tempfile
@@ -74,21 +75,28 @@ def walk(top, topdown=True, followlinks=False):
         yield top, dirs, nondirs
 
 
-def get_matching_docs(dirname, suffix, exclude=(), prune=()):
+def get_matching_docs(dirname, suffix, exclude_docs=(), exclude_dirs=(),
+                      exclude_trees=(), exclude_dirnames=()):
     """
     Get all file names (without suffix) matching a suffix in a
     directory, recursively.
 
-    Exclude files in *exclude*, prune directories in *prune*.
+    Exclude docs in *exclude_docs*, exclude dirs in *exclude_dirs*,
+    prune dirs in *exclude_trees*, prune dirnames in *exclude_dirnames*.
     """
     pattern = '*' + suffix
     # dirname is a normalized absolute path.
     dirname = path.normpath(path.abspath(dirname))
     dirlen = len(dirname) + 1    # exclude slash
     for root, dirs, files in walk(dirname, followlinks=True):
+        if root[dirlen:] in exclude_dirs:
+            continue
+        if root[dirlen:] in exclude_trees:
+            del dirs[:]
+            continue
         dirs.sort()
         files.sort()
-        for prunedir in prune:
+        for prunedir in exclude_dirnames:
             if prunedir in dirs:
                 dirs.remove(prunedir)
         for sfile in files:
@@ -96,7 +104,7 @@ def get_matching_docs(dirname, suffix, exclude=(), prune=()):
                 continue
             qualified_name = path.join(root[dirlen:], sfile[:-len(suffix)])
             qualified_name = qualified_name.replace(os.path.sep, SEP)
-            if qualified_name in exclude:
+            if qualified_name in exclude_docs:
                 continue
             yield qualified_name
 
@@ -156,7 +164,7 @@ def fmt_ex(ex):
 
 
 def rpartition(s, t):
-    """Similar to str.rpartition from 2.5."""
+    """Similar to str.rpartition from 2.5, but doesn't return the separator."""
     i = s.rfind(t)
     if i != -1:
         return s[:i], s[i+len(t):]
@@ -185,3 +193,70 @@ def save_traceback():
     os.write(fd, exc)
     os.close(fd)
     return path
+
+
+def _translate_pattern(pat):
+    """
+    Translate a shell-style glob pattern to a regular expression.
+
+    Adapted from the fnmatch module, but enhanced so that single stars don't
+    match slashes.
+    """
+    i, n = 0, len(pat)
+    res = ''
+    while i < n:
+        c = pat[i]
+        i += 1
+        if c == '*':
+            if i < n and pat[i] == '*':
+                # double star matches slashes too
+                i += 1
+                res = res + '.*'
+            else:
+                # single star doesn't match slashes
+                res = res + '[^/]*'
+        elif c == '?':
+            # question mark doesn't match slashes too
+            res = res + '[^/]'
+        elif c == '[':
+            j = i
+            if j < n and pat[j] == '!':
+                j += 1
+            if j < n and pat[j] == ']':
+                j += 1
+            while j < n and pat[j] != ']':
+                j += 1
+            if j >= n:
+                res = res + '\\['
+            else:
+                stuff = pat[i:j].replace('\\', '\\\\')
+                i = j + 1
+                if stuff[0] == '!':
+                    # negative pattern mustn't match slashes too
+                    stuff = '^/' + stuff[1:]
+                elif stuff[0] == '^':
+                    stuff = '\\' + stuff
+                res = '%s[%s]' % (res, stuff)
+        else:
+            res += re.escape(c)
+    return res + '$'
+
+
+_pat_cache = {}
+
+def patfilter(names, pat):
+    """
+    Return the subset of the list NAMES that match PAT.
+    Adapted from fnmatch module.
+    """
+    result = []
+    if not pat in _pat_cache:
+        _pat_cache[pat] = re.compile(_translate_pattern(pat))
+    match = _pat_cache[pat].match
+    return filter(match, names)
+
+
+no_fn_re = re.compile(r'[:/\\?*%|"\'<>. \t]')
+
+def make_filename(string):
+    return no_fn_re.sub('', string)
